#include "SPI.h"
#include "OLED.h"

#ifndef SPI_IS_BUSY
#define SPI_IS_BUSY(SPIx) (((SPIx)->SR & (SPI_SR_TXE | SPI_SR_RXNE)) == 0 || ((SPIx)->SR & SPI_SR_BSY))
#define SPI_WAIT(SPIx)            while (SPI_IS_BUSY(SPIx))
#endif

void OLED_Write8(uint8_t b)
{
   SPI1->DR = b;
   SPI_WAIT(SPI1);
}

void OLED_WriteCmd(uint8_t cmd)
{
   OLED_DC_LOW;
   OLED_CS_LOW;
   OLED_Write8(cmd);
   OLED_CS_HIGH;
}

void OLED_WriteData(uint8_t data)
{
	OLED_DC_HIGH;
	OLED_CS_LOW;
	OLED_Write8(data);
	OLED_CS_HIGH;
}

void OLEDInit_HW(void)
{
   if (!(RCC->APB2ENR & RCC_APB2ENR_SPI1EN))
   {
     RCC->APB2ENR |= RCC_APB2ENR_SPI1EN;
     RCC->APB2RSTR |= RCC_APB2RSTR_SPI1RST;
     RCC->APB2RSTR &= ~RCC_APB2RSTR_SPI1RST;
   }

   SPI1->CR1 = SPI_CR1_BR_1 | SPI_CR1_BR_0;
   SPI1->CR1 |= SPI_CR1_MSTR;
   SPI1->CR1 |= SPI_CR1_SSI | SPI_CR1_SSM;
   SPI1->CR1 |= SPI_CR1_CPHA | SPI_CR1_CPOL;
   SPI1->CR2 = 0;

   SPI1->CR1 |= SPI_CR1_SPE;

   Nucleo_SetPinGPIO(OLED_PIN_SCK, ioPortAlternatePP);
   Nucleo_SetPinAFGPIO(OLED_PIN_SCK, 5);
   Nucleo_SetPinGPIO(OLED_PIN_MOSI, ioPortAlternatePP);
   Nucleo_SetPinAFGPIO(OLED_PIN_MOSI, 5);
   Nucleo_SetPinGPIO(OLED_PIN_CS, ioPortOutputPP);
   GPIOWrite(OLED_PIN_CS, 1);
   Nucleo_SetPinGPIO(OLED_PIN_RESET, ioPortOutputPP);
   GPIOWrite(OLED_PIN_RESET, 1);
   Nucleo_SetPinGPIO(OLED_PIN_DC, ioPortOutputPP);
   GPIOWrite(OLED_PIN_DC, 1);
}
