#include "stm_core.h"

bool Nucleo_SetPinGPIO(GPIO_TypeDef *gpio,uint32_t bitnum,eIoPortModes mode)
{
	uint32_t enr_mask =0;
	uint32_t rstr_mask = 0;

	switch((uint32_t)gpio)
	{
		case (uint32_t)GPIOA:
				enr_mask = RCC_AHB1ENR_GPIOAEN;
				rstr_mask = RCC_AHB1RSTR_GPIOARST;
			break;
		case (uint32_t)GPIOB:
				enr_mask = RCC_AHB1ENR_GPIOBEN;
				rstr_mask = RCC_AHB1RSTR_GPIOBRST;
			break;
		case (uint32_t)GPIOC:
				enr_mask = RCC_AHB1ENR_GPIOCEN;
				rstr_mask = RCC_AHB1RSTR_GPIOCRST;
			break;
		case (uint32_t)GPIOD:
				enr_mask = RCC_AHB1ENR_GPIODEN;
				rstr_mask = RCC_AHB1RSTR_GPIODRST;
			break;
		case (uint32_t)GPIOE:
				enr_mask = RCC_AHB1ENR_GPIOEEN;
				rstr_mask = RCC_AHB1RSTR_GPIOERST;
			break;
#if defined(STM32F411xE)
#else
		case (uint32_t)GPIOF:
				enr_mask = RCC_AHB1ENR_GPIOFEN;
				rstr_mask = RCC_AHB1RSTR_GPIOFRST;
			break;
		case (uint32_t)GPIOG:
				enr_mask = RCC_AHB1ENR_GPIOGEN;
				rstr_mask = RCC_AHB1RSTR_GPIOGRST;
			break;
#endif
		case (uint32_t)GPIOH:
				enr_mask = RCC_AHB1ENR_GPIOHEN;
				rstr_mask = RCC_AHB1RSTR_GPIOHRST;
			break;
	}
	if ((enr_mask == 0)||(rstr_mask==0))
	{
		return false;
	}
	if (!(RCC->AHB1ENR & enr_mask))
	{
		RCC->AHB1ENR |= enr_mask;
		RCC->AHB1RSTR |= rstr_mask;
		RCC->AHB1RSTR &= ~rstr_mask;
	}
	gpio->MODER &= ~(0x03<<(2*bitnum));
	gpio->PUPDR &= ~(0x03<<(2*bitnum));
	gpio->OSPEEDR &= ~(0x03<<(2*bitnum));

	switch(mode)
	{
		case ioPortOutputOC:
		case ioPortOutputPP:
			gpio->MODER |= 0x01 << (2*bitnum);
			gpio->OSPEEDR |= 0x03 << (2*bitnum);
			gpio->PUPDR &= ~(0x03 << (2*bitnum));

			if (mode == ioPortOutputOC)
			{
				gpio->OTYPER |= 0x01 << bitnum;
			}
			else
			{
				gpio->OTYPER &= ~(0x01 << bitnum);
			}
			break;
		case ioPortInputPU:
			gpio->PUPDR &= ~(0x03<<(2*bitnum));
			gpio->PUPDR |= 0x01 << (2*bitnum);
			break;
		case ioPortInputPD:
			gpio->PUPDR &= ~(0x03 << (2*bitnum));
			gpio->PUPDR |= 0x02 << (2*bitnum);
			break;
		case ioPortInputFloat:
			gpio->PUPDR &= ~(0x00000003<<(2*bitnum));
			break;
		case ioPortAlternatePP:
		case ioPortAlternateOC:
			gpio->MODER |= 0x00000002 << (2*bitnum);
			if (mode==ioPortAlternateOC)
			{
				gpio->OTYPER |= 0x00000001 << bitnum;
			}
			else
			{
				gpio->OTYPER &= ~(0x00000001 << bitnum);
			}
			gpio->OSPEEDR |= 0x00000003 << (2*bitnum);
			gpio->PUPDR &= ~(0x00000003 << (2*bitnum));
			break;
		case ioPortAnalog:
			gpio->MODER |= 0x00000003 << (2*bitnum);
			break;
		default:
			return false;
	}
	return true;
}
bool Nucleo_SetPinAFGPIO(GPIO_TypeDef *gpio, uint32_t bitnum, uint32_t afValue)
{
	gpio->AFR[(bitnum < 8) ? 0 : 1] &= ~(0x0f << (4 * (bitnum & 0x07)));
	gpio->AFR[(bitnum < 8) ? 0 : 1] |= ((afValue & 0x0f) << (4 * (bitnum & 0x07)));
	return true;
}

void GPIOToggle(GPIO_TypeDef * gpio,uint32_t bitnum)
{
	gpio->ODR ^= 1 << bitnum;
}
bool GPIORead(GPIO_TypeDef * gpio,uint32_t bitnum)
{
	return ((gpio->IDR & (1<<bitnum)) != 0);
}
void GPIOWrite(GPIO_TypeDef * gpio,uint32_t bitnum, bool state)
{
	gpio->BSRR = (state) ? (1<<(bitnum)) : ((1<<(bitnum))<<16);
}
